# DON'T CHANGE MANUALLY THIS FILE.
# This file is generated from https://github.com/rundeck/rundeck-api-specs
from enum import Enum
from typing import List, Optional
from pydantic import BaseModel, Field, parse_obj_as
from async_rundeck.proto.json_types import (
    Integer,
    Number,
    String,
    Boolean,
    Object,
    File,
)
import json
from enum import Enum
from typing import List, Optional, Union
from pydantic import parse_raw_as, BaseModel, Field
from async_rundeck.proto.json_types import (
    Integer,
    Number,
    String,
    Boolean,
    Object,
    File,
)
from async_rundeck.client import RundeckClient
from async_rundeck.misc import filter_none
from async_rundeck.exceptions import RundeckError, VersionError
from async_rundeck.proto.definitions import (
    User,
    ModifyUserRequest,
    ModifyUserRequest,
    ModifyUserRequest,
)


class UserRoleListResponse(BaseModel):
    roles: List[String] = Field(alias="roles")


async def user_list(session: RundeckClient) -> List["User"]:
    """List user profiles"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url("/api/{version}/user/list", version=session.version)
    async with session.request(
        "GET", url, data=None, params=filter_none(dict())
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(200): List[User]}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def user_profile_get(session: RundeckClient) -> User:
    """Get same user profile data"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url("/api/{version}/user/info", version=session.version)
    async with session.request(
        "GET", url, data=None, params=filter_none(dict())
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(200): User}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def user_profile_update(
    session: RundeckClient, user: "ModifyUserRequest"
) -> User:
    """Modify same user profile data"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url("/api/{version}/user/info", version=session.version)
    async with session.request(
        "POST",
        url,
        data=json.dumps(user) if isinstance(user, dict) else user.json(),
        params=filter_none(dict()),
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(200): User}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def user_profile_get_by_id(session: RundeckClient, user_i_d: String) -> User:
    """Get another user's profile data"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url(
        "/api/{version}/user/info/{userID}", version=session.version, userID=user_i_d
    )
    async with session.request(
        "GET", url, data=None, params=filter_none(dict())
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(200): User}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def user_profile_update_by_id(
    session: RundeckClient, user_i_d: String, user: ModifyUserRequest
) -> User:
    """Modify another user's profile data"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url(
        "/api/{version}/user/info/{userID}", version=session.version, userID=user_i_d
    )
    async with session.request(
        "POST",
        url,
        data=json.dumps(user) if isinstance(user, dict) else user.json(),
        params=filter_none(dict()),
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(200): User}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def user_role_list(session: RundeckClient) -> UserRoleListResponse:
    """List the roles of the authenticated user"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url("/api/{version}/user/roles", version=session.version)
    async with session.request(
        "GET", url, data=None, params=filter_none(dict())
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(200): UserRoleListResponse}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )



from async_rundeck.rundeck import Rundeck, Execution, ExecutionList, Job, Project
from async_rundeck.exceptions import RundeckError, VersionError

__all__ = [
    "Rundeck",
    "RundeckError",
    "VersionError",
    "Execution",
    "ExecutionList",
    "Job",
    "Project",
]



from contextlib import contextmanager
import os
from typing import Any, Dict
from aiohttp import ClientSession, ClientResponse
from async_rundeck.exceptions import RundeckError
from copy import deepcopy


class RundeckClient:
    _default_options = {
        "headers": {
            "Accept": "application/json",
            "Content-Type": "application/json",
        },
    }

    def __init__(
        self,
        url: str = None,
        token: str = None,
        username: str = None,
        password: str = None,
        api_version: int = None,
    ) -> None:
        self.url = url or os.getenv("RUNDECK_URL", "http://localhost:4440")
        self.url = self.url.rstrip("/")
        self.token = token or os.getenv("RUNDECK_TOKEN")
        self.username = username or os.getenv("RUNDECK_USERNAME")
        self.password = password or os.getenv("RUNDECK_PASSWORD")
        self.api_version = api_version or int(os.getenv("RUNDECK_API_VERSION", "32"))
        if self.token is None and (self.password is None or self.password is None):
            raise ValueError("Cannot authenticate without a token or username/password")
        self.session_id: str = None
        self._session: ClientSession = None
        self._context_options: Dict[str, Any] = deepcopy(self._default_options)

    @contextmanager
    def context_options(self, options: Dict[str, Any]) -> Any:
        self._context_options = options
        yield
        self._context_options = deepcopy(self._default_options)

    @property
    def version(self) -> str:
        """Alias of api_version"""
        return self.api_version

    @property
    def options(self) -> Dict[str, Dict[str, Any]]:
        options = self._context_options
        if self.session_id is not None:
            options["cookies"] = {"JSESSIONID": self.session_id}

        return options

    def format_url(self, path: str, **kwargs) -> str:
        """format url with path and query parameters

        Parameters
        ----------
        path : str
            accesing url

        Returns
        -------
        str
            formatted url
        """
        return (self.url + path).format(**kwargs)

    async def __aenter__(self) -> "RundeckClient":
        if self._session is None:
            self._session = ClientSession()
            self._session = await self._session.__aenter__()
            if self.token is None and self.session_id is None:
                self.session_id = await self.auth(self._session)
        return self

    async def __aexit__(self, *args) -> "RundeckClient":
        if self._session is not None:
            await self._session.__aexit__(*args)
        self._session = None

    def request(self, method: str, url: str, **kwargs) -> ClientResponse:
        """Requesting a resource

        Parameters
        ----------
        method : str
            Requesting method
        url : str
            url to request

        Returns
        -------
        ClientResponse
            response object. see aiohttp.ClientResponse for more details
        """
        options = self.options
        for k, v in kwargs.items():
            if isinstance(v, dict):
                options.setdefault(k, {}).update(v)
            else:
                options[k] = v
        if self.token:
            options["headers"]["X-Rundeck-Auth-Token"] = self.token

        return self._session.request(method, url, **options)

    async def auth(self, session: ClientSession) -> str:
        """username/password authentication

        Parameters
        ----------
        session : ClientSession
            session to use for authentication

        Returns
        -------
        str
            session id

        Raises
        ------
        RundeckError
            if authentication fails
        """
        url = self.url + "/j_security_check"
        p = {"j_username": self.username, "j_password": self.password}
        async with session.post(
            url,
            data=p,
        ) as response:
            if len(response.history) > 1:
                async with response.history[0] as r:
                    session_id = r.cookies.get("JSESSIONID")
            else:
                session_id = response.cookies.get("JSESSIONID")
        if session_id is None:
            raise RundeckError("Authrorization failed")
        return session_id.value



# DON'T CHANGE MANUALLY THIS FILE.
# This file is generated from https://github.com/rundeck/rundeck-api-specs
from enum import Enum
from typing import List, Optional, Union
from pydantic import BaseModel, Field
from async_rundeck.proto.json_types import (
    Integer,
    Number,
    String,
    Boolean,
    Object,
    File,
)


class User(BaseModel):
    login: Optional[String] = Field(alias="login")
    first_name: Optional[String] = Field(alias="firstName")
    last_name: Optional[String] = Field(alias="lastName")
    email: Optional[String] = Field(alias="email")


class ModifyUserRequest(BaseModel):
    first_name: Optional[String] = Field(alias="firstName")
    last_name: Optional[String] = Field(alias="lastName")
    email: Optional[String] = Field(alias="email")


class Timestamp(BaseModel):
    epoch: Optional[Number] = Field(alias="epoch")
    unit: Optional[String] = Field(alias="unit")
    datetime: Optional[String] = Field(alias="datetime")


class Rundeck(BaseModel):
    version: Optional[String] = Field(alias="version")
    build: Optional[String] = Field(alias="build")
    node: Optional[String] = Field(alias="node")
    base: Optional[String] = Field(alias="base")
    apiversion: Optional[Number] = Field(alias="apiversion")
    server_uuid: Optional[String] = Field(alias="serverUUID")


class Executions(BaseModel):
    active: Optional[Boolean] = Field(alias="active")
    execution_mode: Optional[String] = Field(alias="executionMode")


class Os(BaseModel):
    arch: Optional[String] = Field(alias="arch")
    name: Optional[String] = Field(alias="name")
    version: Optional[String] = Field(alias="version")


class Jvm(BaseModel):
    name: Optional[String] = Field(alias="name")
    vendor: Optional[String] = Field(alias="vendor")
    version: Optional[String] = Field(alias="version")
    implementation_version: Optional[String] = Field(alias="implementationVersion")


class Since(BaseModel):
    epoch: Optional[Number] = Field(alias="epoch")
    unit: Optional[String] = Field(alias="unit")
    datetime: Optional[String] = Field(alias="datetime")


class Uptime(BaseModel):
    duration: Optional[Number] = Field(alias="duration")
    unit: Optional[String] = Field(alias="unit")
    since: Optional[Since] = Field(alias="since")


class LoadAverage(BaseModel):
    unit: Optional[String] = Field(alias="unit")
    average: Optional[Number] = Field(alias="average")


class Cpu(BaseModel):
    load_average: Optional[LoadAverage] = Field(alias="loadAverage")
    processors: Optional[Number] = Field(alias="processors")


class Memory(BaseModel):
    unit: Optional[String] = Field(alias="unit")
    max: Optional[Number] = Field(alias="max")
    free: Optional[Number] = Field(alias="free")
    total: Optional[Number] = Field(alias="total")


class Scheduler(BaseModel):
    running: Optional[Number] = Field(alias="running")
    thread_pool_size: Optional[Number] = Field(alias="threadPoolSize")


class Threads(BaseModel):
    active: Optional[Number] = Field(alias="active")


class Stats(BaseModel):
    uptime: Optional[Uptime] = Field(alias="uptime")
    cpu: Optional[Cpu] = Field(alias="cpu")
    memory: Optional[Memory] = Field(alias="memory")
    scheduler: Optional[Scheduler] = Field(alias="scheduler")
    threads: Optional[Threads] = Field(alias="threads")


class Metrics(BaseModel):
    href: Optional[String] = Field(alias="href")
    content_type: Optional[String] = Field(alias="contentType")


class ThreadDump(BaseModel):
    href: Optional[String] = Field(alias="href")
    content_type: Optional[String] = Field(alias="contentType")


class System(BaseModel):
    timestamp: Optional[Timestamp] = Field(alias="timestamp")
    rundeck: Optional[Rundeck] = Field(alias="rundeck")
    executions: Optional[Executions] = Field(alias="executions")
    os: Optional[Os] = Field(alias="os")
    jvm: Optional[Jvm] = Field(alias="jvm")
    stats: Optional[Stats] = Field(alias="stats")
    metrics: Optional[Metrics] = Field(alias="metrics")
    thread_dump: Optional[ThreadDump] = Field(alias="threadDump")


class SystemInfo(BaseModel):
    system: Optional[System] = Field(alias="system")


class LogStorage(BaseModel):
    enabled: Optional[Boolean] = Field(alias="enabled")
    plugin_name: Optional[String] = Field(alias="pluginName")
    succeeded_count: Optional[Number] = Field(alias="succeededCount")
    failed_count: Optional[Number] = Field(alias="failedCount")
    queued_count: Optional[Number] = Field(alias="queuedCount")
    total_count: Optional[Number] = Field(alias="totalCount")
    incomplete_count: Optional[Number] = Field(alias="incompleteCount")
    missing_count: Optional[Number] = Field(alias="missingCount")


class Storage(BaseModel):
    local_files_present: Optional[Boolean] = Field(alias="localFilesPresent")
    incomplete_filetypes: Optional[List[String]] = Field(alias="incompleteFiletypes")
    queued: Optional[Boolean] = Field(alias="queued")
    failed: Optional[Boolean] = Field(alias="failed")
    date: Optional[String] = Field(alias="date")


class IncompleteLogExecution(BaseModel):
    id: Optional[String] = Field(alias="id")
    project: Optional[String] = Field(alias="project")
    href: Optional[String] = Field(alias="href")
    permalink: Optional[String] = Field(alias="permalink")
    storage: Optional[Storage] = Field(alias="storage")
    errors: Optional[List[String]] = Field(alias="errors")


class IncompleteFileType(Enum):
    Rdlog = "rdlog"
    Statson = "state.json"
    Executioml = "execution.xml"


class IncompleteLogExecutions(BaseModel):
    total: Number = Field(alias="total")
    max: Number = Field(alias="max")
    offset: Number = Field(alias="offset")
    executions: List[IncompleteLogExecution] = Field(alias="executions")


class Server(BaseModel):
    uuid: Optional[String] = Field(alias="uuid")


class Job(BaseModel):
    id: Optional[String] = Field(alias="id")
    name: Optional[String] = Field(alias="name")
    group: Optional[String] = Field(alias="group")
    project: Optional[String] = Field(alias="project")
    description: Optional[String] = Field(alias="description")
    href: Optional[String] = Field(alias="href")
    permalink: Optional[String] = Field(alias="permalink")
    scheduled: Optional[Boolean] = Field(alias="scheduled")
    schedule_enabled: Optional[Boolean] = Field(alias="scheduleEnabled")
    server_node_uuid: Optional[String] = Field(alias="serverNodeUUID")
    server_owner: Optional[String] = Field(alias="serverOwner")
    enabled: Optional[Boolean] = Field(alias="enabled")


class TakeoverScheduleRequest(BaseModel):
    server: Optional[Server] = Field(alias="server")
    project: Optional[String] = Field(alias="project")
    job: Optional[Job] = Field(alias="job")


class JobReference(BaseModel):
    href: Optional[String] = Field(alias="href")
    permalink: Optional[String] = Field(alias="permalink")
    id: Optional[String] = Field(alias="id")
    pervious_owner: Optional[String] = Field(alias="pervious-owner")


class Jobs(BaseModel):
    failed: Optional[List[JobReference]] = Field(alias="failed")
    successfull: Optional[List[JobReference]] = Field(alias="successfull")
    total: Optional[Number] = Field(alias="total")


class TakeoverSchedule(BaseModel):
    project: Optional[String] = Field(alias="project")
    jobs: Optional[Jobs] = Field(alias="jobs")
    server: Optional[Server] = Field(alias="server")


class Self(BaseModel):
    server: Optional[Server] = Field(alias="server")


class TakeoverScheduleResponse(BaseModel):
    takeover_schedule: Optional[TakeoverSchedule] = Field(alias="takeoverSchedule")
    self: Optional[Self] = Field(alias="self")
    message: Optional[String] = Field(alias="message")
    apiversion: Optional[Number] = Field(alias="apiversion")
    success: Optional[Boolean] = Field(alias="success")


class Project(BaseModel):
    description: Optional[String] = Field(alias="description")
    name: Optional[String] = Field(alias="name")
    url: Optional[String] = Field(alias="url")
    config: Optional[Object] = Field(alias="config")


class JobMetadata(BaseModel):
    id: Optional[String] = Field(alias="id")
    name: Optional[String] = Field(alias="name")
    group: Optional[String] = Field(alias="group")
    project: Optional[String] = Field(alias="project")
    description: Optional[String] = Field(alias="description")
    href: Optional[String] = Field(alias="href")
    permalink: Optional[String] = Field(alias="permalink")
    scheduled: Optional[Boolean] = Field(alias="scheduled")
    schedule_enabled: Optional[Boolean] = Field(alias="scheduleEnabled")
    average_duration: Optional[Number] = Field(alias="averageDuration")
    options: Optional[Object] = Field(alias="options")


class JobBulkOperationResponse(BaseModel):
    request_count: Number = Field(alias="requestCount")
    allsuccessful: Boolean = Field(alias="allsuccessful")
    succeeded: Optional[List["BulkJobSucceededInfo"]] = Field(alias="succeeded")
    failed: Optional[List["BulkJobFailedInfo"]] = Field(alias="failed")


class JobInputFileUploadResponse(BaseModel):
    total: Number = Field(alias="total")
    options: Object = Field(alias="options")


class JobInputFileListResponse(BaseModel):
    paging: "Paging" = Field(alias="paging")
    files: List["JobInputFileInfo"] = Field(alias="files")


class FileState(Enum):
    Temp = "temp"
    Deleted = "deleted"
    Expired = "expired"
    Retained = "retained"


class JobInputFileInfo(BaseModel):
    id: String = Field(alias="id")
    user: String = Field(alias="user")
    file_state: FileState = Field(alias="fileState")
    sha: String = Field(alias="sha")
    job_id: String = Field(alias="jobId")
    date_created: String = Field(alias="dateCreated")
    server_node_uuid: String = Field(alias="serverNodeUUID")
    file_name: Optional[String] = Field(alias="fileName")
    size: Optional[Integer] = Field(alias="size")
    expiration_date: String = Field(alias="expirationDate")
    exec_id: Optional[String] = Field(alias="execId")


class BulkJobSucceededInfo(BaseModel):
    id: String = Field(alias="id")
    message: String = Field(alias="message")


class BulkJobFailedInfo(BaseModel):
    id: String = Field(alias="id")
    error_code: String = Field(alias="errorCode")
    message: String = Field(alias="message")


class ExecutionList(BaseModel):
    paging: Optional["Paging"] = Field(alias="paging")
    executions: Optional[List["Execution"]] = Field(alias="executions")


class Status(Enum):
    Running = "running"
    Succeeded = "succeeded"
    Failed = "failed"
    Aborted = "aborted"
    Timedout = "timedout"
    Faileitetry = "failed-with-retry"
    Scheduled = "scheduled"
    Other = "other"


class Dattarted(BaseModel):
    unixtime: Optional[Number] = Field(alias="unixtime")
    date: Optional[String] = Field(alias="date")


class Execution(BaseModel):
    id: Optional[String] = Field(alias="id")
    href: Optional[String] = Field(alias="href")
    permalink: Optional[String] = Field(alias="permalink")
    status: Optional[Status] = Field(alias="status")
    custom_status: Optional[String] = Field(alias="customStatus")
    project: Optional[String] = Field(alias="project")
    user: Optional[String] = Field(alias="user")
    server_uuid: Optional[String] = Field(alias="serverUUID")
    date_started: Optional[Dattarted] = Field(alias="date-started")
    job: Optional[JobMetadata] = Field(alias="job")
    description: Optional[String] = Field(alias="description")
    argstring: Optional[String] = Field(alias="argstring")
    successful_nodes: Optional[List[String]] = Field(alias="successfulNodes")


class JobExecutionDelete(BaseModel):
    failed_count: Optional[Number] = Field(alias="failedCount")
    success_count: Optional[Number] = Field(alias="successCount")
    allsuccessful: Optional[Boolean] = Field(alias="allsuccessful")
    request_count: Optional[Number] = Field(alias="requestCount")
    failures: Optional[List[Object]] = Field(alias="failures")


class AclReference(BaseModel):
    path: Optional[String] = Field(alias="path")
    type: Optional[String] = Field(alias="type")
    name: Optional[String] = Field(alias="name")
    href: Optional[String] = Field(alias="href")


class AclList(BaseModel):
    path: Optional[String] = Field(alias="path")
    type: Optional[String] = Field(alias="type")
    href: Optional[String] = Field(alias="href")
    resources: Optional[List[AclReference]] = Field(alias="resources")


class AclPolicyResponse(BaseModel):
    contents: String = Field(alias="contents")


class InvalidAclPolicyResponse(BaseModel):
    valid: Optional[Boolean] = Field(alias="valid")
    policies: Optional[List[Object]] = Field(alias="policies")


class Loglevel(Enum):
    Debug = "DEBUG"
    Verbose = "VERBOSE"
    Info = "INFO"
    Warn = "WARN"
    Error = "ERROR"


class RetryExecutionRequest(BaseModel):
    arg_string: Optional[String] = Field(alias="argString")
    log_level: Optional[Loglevel] = Field(alias="loglevel")
    as_user: Optional[String] = Field(alias="asUser")
    options: Optional[Object] = Field(alias="options")


class ExecuteJobRequest(BaseModel):
    arg_string: Optional[String] = Field(alias="argString")
    log_level: Optional[Loglevel] = Field(alias="loglevel")
    as_user: Optional[String] = Field(alias="asUser")
    filter: Optional[String] = Field(alias="filter")
    run_at_time: Optional[String] = Field(alias="runAtTime")
    options: Optional[Object] = Field(alias="options")


class StorageKeyMetaType(Enum):
    Private = "private"
    Public = "public"


class Meta(BaseModel):
    rundeck_key_type: Optional[StorageKeyMetaType] = Field(alias="Rundeck-key-type")
    rundeck_content_mask: Optional[String] = Field(alias="Rundeck-content-mask")
    rundeck_content_size: Optional[String] = Field(alias="Rundeck-content-size")
    rundeck_content_type: Optional[String] = Field(alias="Rundeck-content-type")


class StorageKeyMetadata(BaseModel):
    meta: Optional[Meta] = Field(alias="meta")
    url: Optional[String] = Field(alias="url")
    name: Optional[String] = Field(alias="name")
    type: Optional[String] = Field(alias="type")
    path: Optional[String] = Field(alias="path")


class StorageKeyListResponse(BaseModel):
    resources: Optional[List[StorageKeyMetadata]] = Field(alias="resources")
    meta: Optional[Meta] = Field(alias="meta")
    url: Optional[String] = Field(alias="url")
    type: Optional[String] = Field(alias="type")
    path: Optional[String] = Field(alias="path")


class Paging(BaseModel):
    count: Optional[Integer] = Field(alias="count")
    total: Optional[Integer] = Field(alias="total")
    offset: Optional[Integer] = Field(alias="offset")
    max: Optional[Integer] = Field(alias="max")


class ExecutionOutput(BaseModel):
    id: String = Field(alias="id")
    error: Optional[String] = Field(alias="error")
    empty: Optional[Boolean] = Field(alias="empty")
    offset: String = Field(alias="offset")
    completed: Boolean = Field(alias="completed")
    exec_completed: Boolean = Field(alias="execCompleted")
    has_failed_nodes: Boolean = Field(alias="hasFailedNodes")
    exec_state: String = Field(alias="execState")
    last_modified: String = Field(alias="lastModified")
    exec_duration: Number = Field(alias="execDuration")
    percent_loaded: Optional[Number] = Field(alias="percentLoaded")
    total_size: Number = Field(alias="totalSize")
    retry_backoff: Number = Field(alias="retryBackoff")
    cluster_exec: Boolean = Field(alias="clusterExec")
    compacted: Boolean = Field(alias="compacted")
    entries: List["ExecutionOutputEntry"] = Field(alias="entries")


class ExecutionState(BaseModel):
    execution_id: String = Field(alias="executionId")
    server_node: String = Field(alias="serverNode")
    execution_state: Optional["ExecutionStateState"] = Field(alias="executionState")
    completed: Boolean = Field(alias="completed")
    target_nodes: Optional[List[String]] = Field(alias="targetNodes")
    all_nodes: Optional[List[String]] = Field(alias="allNodes")
    step_count: Optional[Number] = Field(alias="stepCount")
    update_time: Optional[String] = Field(alias="updateTime")
    start_time: Optional[String] = Field(alias="startTime")
    end_time: Optional[String] = Field(alias="endTime")
    nodes: Optional[Object] = Field(alias="nodes")
    steps: Optional[List["ExecutionStateStep"]] = Field(alias="steps")


class ExecutionStateState(Enum):
    Succeeded = "SUCCEEDED"
    Failed = "FAILED"
    Running = "RUNNING"
    Waiting = "WAITING"


class ExecutionStateWorkflow(BaseModel):
    completed: Boolean = Field(alias="completed")
    update_time: Optional[String] = Field(alias="updateTime")
    start_time: Optional[String] = Field(alias="startTime")
    end_time: Optional[String] = Field(alias="endTime")
    step_count: Optional[Number] = Field(alias="stepCount")
    target_nodes: Optional[List[String]] = Field(alias="targetNodes")
    all_nodes: Optional[List[String]] = Field(alias="allNodes")
    steps: Optional[List["ExecutionStateStep"]] = Field(alias="steps")


class ExecutionStateNodeState(BaseModel):
    execution_state: ExecutionStateState = Field(alias="executionState")
    stepctx: String = Field(alias="stepctx")


class ExecutionStateStep(BaseModel):
    id: String = Field(alias="id")
    stepctx: String = Field(alias="stepctx")
    node_step: Boolean = Field(alias="nodeStep")
    execution_state: ExecutionStateState = Field(alias="executionState")
    duration: Number = Field(alias="duration")
    start_time: Optional[String] = Field(alias="startTime")
    end_time: Optional[String] = Field(alias="endTime")
    update_time: Optional[String] = Field(alias="updateTime")
    node_states: Optional[Object] = Field(alias="nodeStates")
    has_subworkflow: Optional[Boolean] = Field(alias="hasSubworkflow")
    workflow: Optional[ExecutionStateWorkflow] = Field(alias="workflow")
    parameters: Optional[Object] = Field(alias="parameters")
    parameter_states: Optional[Object] = Field(alias="parameterStates")


class ExecutionStateStepNodeState(BaseModel):
    execution_state: ExecutionStateState = Field(alias="executionState")
    duration: Number = Field(alias="duration")
    start_time: Optional[String] = Field(alias="startTime")
    update_time: Optional[String] = Field(alias="updateTime")
    end_time: Optional[String] = Field(alias="endTime")


class ExecutionOutputEntry(BaseModel):
    time: Optional[String] = Field(alias="time")
    absolute_time: Optional[String] = Field(alias="absolute_time")
    log: Optional[String] = Field(alias="log")
    level: Optional[String] = Field(alias="level")
    stepctx: Optional[String] = Field(alias="stepctx")
    node: Optional[String] = Field(alias="node")


class Jobref(BaseModel):
    name: Optional[String] = Field(alias="name")
    group: Optional[String] = Field(alias="group")
    uuid: Optional[String] = Field(alias="uuid")
    node_step: Optional[String] = Field(alias="nodeStep")
    import_options: Optional[Boolean] = Field(alias="importOptions")


class WorkflowStep(BaseModel):
    jobref: Optional[Jobref] = Field(alias="jobref")
    job_id: Optional[String] = Field(alias="jobId")
    description: Optional[String] = Field(alias="description")
    exec: Optional[String] = Field(alias="exec")
    script: Optional[String] = Field(alias="script")
    scriptfile: Optional[String] = Field(alias="scriptfile")
    scripturl: Optional[String] = Field(alias="scripturl")
    type: Optional[String] = Field(alias="type")
    node_step: Optional[String] = Field(alias="nodeStep")
    workflow: Optional[List["WorkflowStep"]] = Field(alias="workflow")



# DON'T CHANGE MANUALLY THIS FILE.
# This file is generated from https://github.com/rundeck/rundeck-api-specs
from enum import Enum
from typing import List, Optional
from pydantic import BaseModel, Field, parse_obj_as
from async_rundeck.proto.json_types import (
    Integer,
    Number,
    String,
    Boolean,
    Object,
    File,
)
import json
from enum import Enum
from typing import List, Optional, Union
from pydantic import parse_raw_as, BaseModel, Field
from async_rundeck.proto.json_types import (
    Integer,
    Number,
    String,
    Boolean,
    Object,
    File,
)
from async_rundeck.client import RundeckClient
from async_rundeck.misc import filter_none
from async_rundeck.exceptions import RundeckError, VersionError
from async_rundeck.proto.definitions import (
    Job,
    ExecutionList,
    Execution,
    ExecuteJobRequest,
    ExecuteJobRequest,
    JobExecutionDelete,
    RetryExecutionRequest,
    RetryExecutionRequest,
    JobMetadata,
    JobBulkOperationResponse,
    JobInputFileUploadResponse,
    File,
    JobInputFileListResponse,
    JobInputFileInfo,
    WorkflowStep,
)


class JobBulkDeleteRequest(BaseModel):
    ids: List[String] = Field(alias="ids")


class JobExecutionEnableResponse(BaseModel):
    success: Optional[Boolean] = Field(alias="success")


class JobExecutionDisableResponse(BaseModel):
    success: Optional[Boolean] = Field(alias="success")


class JobScheduleEnableResponse(BaseModel):
    success: Optional[Boolean] = Field(alias="success")


class JobScheduleDisableResponse(BaseModel):
    success: Optional[Boolean] = Field(alias="success")


class JobExecutionBulkEnableRequest(BaseModel):
    ids: List[String] = Field(alias="ids")


class JobExecutionBulkDisableRequest(BaseModel):
    ids: List[String] = Field(alias="ids")


class JobScheduleBulkEnableRequest(BaseModel):
    ids: List[String] = Field(alias="ids")


class JobScheduleBulkDisableRequest(BaseModel):
    ids: List[String] = Field(alias="ids")


class JobWorkflowGetResponse(BaseModel):
    workflow: List["WorkflowStep"] = Field(alias="workflow")


async def job_list(
    session: RundeckClient,
    project: String,
    *,
    id_list: Optional[String] = None,
    group_path: Optional[String] = "*",
    job_filter: Optional[String] = None,
    job_exact_filter: Optional[String] = None,
    group_path_exact: Optional[String] = None,
    scheduled_filter: Optional[Boolean] = None,
    server_node_uuid_filter: Optional[String] = None,
) -> List["Job"]:
    """List the jobs that exist for a project"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url(
        "/api/{version}/project/{project}/jobs",
        version=session.version,
        project=project,
    )
    async with session.request(
        "GET",
        url,
        data=None,
        params=filter_none(
            dict(
                idList=id_list,
                groupPath=group_path,
                jobFilter=job_filter,
                jobExactFilter=job_exact_filter,
                groupPathExact=group_path_exact,
                scheduledFilter=scheduled_filter,
                serverNodeUUIDFilter=server_node_uuid_filter,
            )
        ),
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(200): List[Job]}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def job_execution_list(session: RundeckClient, id: String) -> ExecutionList:
    """List job executions"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url(
        "/api/{version}/job/{id}/executions", version=session.version, id=id
    )
    async with session.request(
        "GET", url, data=None, params=filter_none(dict())
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(200): ExecutionList}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def job_execution_run(
    session: RundeckClient, id: String, *, request: Optional["ExecuteJobRequest"] = None
) -> Execution:
    """Run the specified job"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url(
        "/api/{version}/job/{id}/executions", version=session.version, id=id
    )
    async with session.request(
        "POST",
        url,
        data=json.dumps(request) if isinstance(request, dict) else request.json(),
        params=filter_none(dict()),
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(200): Execution}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def job_execution_delete(
    session: RundeckClient, id: Integer
) -> JobExecutionDelete:
    """Delete all job executions"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url(
        "/api/{version}/job/{id}/executions", version=session.version, id=id
    )
    async with session.request(
        "DELETE", url, data=None, params=filter_none(dict())
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(204): JobExecutionDelete}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def job_retry_execution(
    session: RundeckClient,
    job_id: String,
    execution_id: Integer,
    *,
    request: Optional["RetryExecutionRequest"] = None,
) -> ExecutionList:
    """Retry a failed job execution on failed nodes only or on the same as the execution. This is the same functionality as the `Retry Failed Nodes ...` button on the execution page."""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url(
        "/api/{version}/job/{jobID}/retry/{executionID}",
        version=session.version,
        jobID=job_id,
        executionID=execution_id,
    )
    async with session.request(
        "POST",
        url,
        data=json.dumps(request) if isinstance(request, dict) else request.json(),
        params=filter_none(dict()),
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(200): ExecutionList}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def job_get(
    session: RundeckClient, id: String, *, format: Optional[String] = "xml"
) -> Union["File", None]:
    """Export a single job definition in XML or YAML formats."""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url("/api/{version}/job/{id}", version=session.version, id=id)
    async with session.request(
        "GET", url, data=None, params=filter_none(dict(format=format))
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(200): File, (404): None}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def job_delete(session: RundeckClient, id: String) -> Union[None, None]:
    """Delete a single job definition."""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url("/api/{version}/job/{id}", version=session.version, id=id)
    async with session.request(
        "DELETE", url, data=None, params=filter_none(dict())
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(204): None, (404): None}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def job_info_get(session: RundeckClient, id: String) -> JobMetadata:
    """Get metadata about a specific job."""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url(
        "/api/{version}/job/{id}/info", version=session.version, id=id
    )
    async with session.request(
        "GET", url, data=None, params=filter_none(dict())
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(200): JobMetadata}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def job_bulk_delete(
    session: RundeckClient, job_bulk_delete_request: JobBulkDeleteRequest
) -> JobBulkOperationResponse:
    """Delete multiple job definitions at once"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url("/api/{version}/jobs/delete", version=session.version)
    async with session.request(
        "POST",
        url,
        data=json.dumps(job_bulk_delete_request)
        if isinstance(job_bulk_delete_request, dict)
        else job_bulk_delete_request.json(),
        params=filter_none(dict()),
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(200): JobBulkOperationResponse}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def job_execution_enable(
    session: RundeckClient, id: String
) -> JobExecutionEnableResponse:
    """Enable executions for a job. (ACL requires toggle_execution action for a job.)"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url(
        "/api/{version}/job/{id}/execution/enable", version=session.version, id=id
    )
    async with session.request(
        "POST", url, data=None, params=filter_none(dict())
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(200): JobExecutionEnableResponse}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def job_execution_disable(
    session: RundeckClient, id: String
) -> JobExecutionDisableResponse:
    """Disable all executions for a job (scheduled or manual). (ACL requires toggle_execution action for a job.)"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url(
        "/api/{version}/job/{id}/execution/disable", version=session.version, id=id
    )
    async with session.request(
        "POST", url, data=None, params=filter_none(dict())
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(200): JobExecutionDisableResponse}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def job_schedule_enable(
    session: RundeckClient, id: String
) -> JobScheduleEnableResponse:
    """Enable the schedule for a job. (ACL requires toggle_schedule action for a job.)"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url(
        "/api/{version}/job/{id}/schedule/enable", version=session.version, id=id
    )
    async with session.request(
        "POST", url, data=None, params=filter_none(dict())
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(200): JobScheduleEnableResponse}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def job_schedule_disable(
    session: RundeckClient, id: String
) -> JobScheduleDisableResponse:
    """Disable the schedule for a job. (ACL requires toggle_schedule action for a job.)"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url(
        "/api/{version}/job/{id}/schedule/disable", version=session.version, id=id
    )
    async with session.request(
        "POST", url, data=None, params=filter_none(dict())
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(200): JobScheduleDisableResponse}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def job_execution_bulk_enable(
    session: RundeckClient,
    job_execution_bulk_enable_request: JobExecutionBulkEnableRequest,
) -> JobBulkOperationResponse:
    """Bulk enable job executions"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url(
        "/api/{version}/jobs/execution/enable", version=session.version
    )
    async with session.request(
        "POST",
        url,
        data=json.dumps(job_execution_bulk_enable_request)
        if isinstance(job_execution_bulk_enable_request, dict)
        else job_execution_bulk_enable_request.json(),
        params=filter_none(dict()),
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(200): JobBulkOperationResponse}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def job_execution_bulk_disable(
    session: RundeckClient,
    job_execution_bulk_disable_request: JobExecutionBulkDisableRequest,
) -> JobBulkOperationResponse:
    """Bulk disable job executions"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url(
        "/api/{version}/jobs/execution/disable", version=session.version
    )
    async with session.request(
        "POST",
        url,
        data=json.dumps(job_execution_bulk_disable_request)
        if isinstance(job_execution_bulk_disable_request, dict)
        else job_execution_bulk_disable_request.json(),
        params=filter_none(dict()),
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(200): JobBulkOperationResponse}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def job_schedule_bulk_enable(
    session: RundeckClient,
    job_schedule_bulk_enable_request: JobScheduleBulkEnableRequest,
) -> JobBulkOperationResponse:
    """Bulk enable job schedule"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url(
        "/api/{version}/jobs/schedule/enable", version=session.version
    )
    async with session.request(
        "POST",
        url,
        data=json.dumps(job_schedule_bulk_enable_request)
        if isinstance(job_schedule_bulk_enable_request, dict)
        else job_schedule_bulk_enable_request.json(),
        params=filter_none(dict()),
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(200): JobBulkOperationResponse}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def job_schedule_bulk_disable(
    session: RundeckClient,
    job_schedule_bulk_disable_request: JobScheduleBulkDisableRequest,
) -> JobBulkOperationResponse:
    """Bulk disable job schedule"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url(
        "/api/{version}/jobs/schedule/disable", version=session.version
    )
    async with session.request(
        "POST",
        url,
        data=json.dumps(job_schedule_bulk_disable_request)
        if isinstance(job_schedule_bulk_disable_request, dict)
        else job_schedule_bulk_disable_request.json(),
        params=filter_none(dict()),
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(200): JobBulkOperationResponse}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def job_input_file_upload(
    session: RundeckClient,
    id: String,
    option_name: String,
    file_name: String,
    file: "File",
) -> JobInputFileUploadResponse:
    """Upload file as job option"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url(
        "/api/{version}/job/{id}/input/file", version=session.version, id=id
    )
    async with session.request(
        "POST",
        url,
        data=file,
        params=filter_none(dict(optionName=option_name, fileName=file_name)),
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(200): JobInputFileUploadResponse}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def job_input_file_list(
    session: RundeckClient, id: String
) -> JobInputFileListResponse:
    """List uploaded input files for job"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url(
        "/api/{version}/job/{id}/input/files", version=session.version, id=id
    )
    async with session.request(
        "GET", url, data=None, params=filter_none(dict())
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(200): JobInputFileListResponse}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def job_input_file_info_get(
    session: RundeckClient, id: String
) -> JobInputFileInfo:
    """Get job input file info"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url(
        "/api/{version}/jobs/file/{id}", version=session.version, id=id
    )
    async with session.request(
        "GET", url, data=None, params=filter_none(dict())
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(200): JobInputFileInfo}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def job_workflow_get(
    session: RundeckClient, id: String
) -> JobWorkflowGetResponse:
    """Get job workflow tree."""
    if session.version < 34:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url(
        "/api/{version}/job/{id}/workflow", version=session.version, id=id
    )
    async with session.request(
        "GET", url, data=None, params=filter_none(dict())
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(200): JobWorkflowGetResponse}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )



# DON'T CHANGE MANUALLY THIS FILE.
# This file is generated from https://github.com/rundeck/rundeck-api-specs
from enum import Enum
from typing import List, Optional
from pydantic import BaseModel, Field, parse_obj_as
from async_rundeck.proto.json_types import (
    Integer,
    Number,
    String,
    Boolean,
    Object,
    File,
)
import json
from enum import Enum
from typing import List, Optional, Union
from pydantic import parse_raw_as, BaseModel, Field
from async_rundeck.proto.json_types import (
    Integer,
    Number,
    String,
    Boolean,
    Object,
    File,
)
from async_rundeck.client import RundeckClient
from async_rundeck.misc import filter_none
from async_rundeck.exceptions import RundeckError, VersionError
from async_rundeck.proto.definitions import (
    Execution,
    JobExecutionDelete,
    ExecutionState,
    JobInputFileInfo,
    ExecutionList,
    ExecutionOutput,
)


class ExecutionBulkDeleteRequest(BaseModel):
    ids: List[String] = Field(alias="ids")


class ExecutionInputFilesListResponse(BaseModel):
    files: List["JobInputFileInfo"] = Field(alias="files")


async def execution_status_get(session: RundeckClient, id: String) -> Execution:
    """Get the status of an execution by ID"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url(
        "/api/{version}/execution/{id}", version=session.version, id=id
    )
    async with session.request(
        "GET", url, data=None, params=filter_none(dict())
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(200): Execution}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def execution_delete(session: RundeckClient, id: String) -> None:
    """Delete an exeuction by ID"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url(
        "/api/{version}/execution/{id}", version=session.version, id=id
    )
    async with session.request(
        "DELETE", url, data=None, params=filter_none(dict())
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(204): None}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def execution_bulk_delete(
    session: RundeckClient, execution_bulk_delete_request: ExecutionBulkDeleteRequest
) -> JobExecutionDelete:
    """Bulk delete executions"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url(
        "/api/{version}/executions/delete", version=session.version
    )
    async with session.request(
        "POST",
        url,
        data=json.dumps(execution_bulk_delete_request)
        if isinstance(execution_bulk_delete_request, dict)
        else execution_bulk_delete_request.json(),
        params=filter_none(dict()),
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(200): JobExecutionDelete}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def execution_state_get(session: RundeckClient, id: String) -> ExecutionState:
    """Get detail about the node and step state of an execution by ID. The execution can be currently running or completed."""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url(
        "/api/{version}/execution/{id}/state", version=session.version, id=id
    )
    async with session.request(
        "GET", url, data=None, params=filter_none(dict())
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(200): ExecutionState}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def execution_input_files_list(
    session: RundeckClient, id: String
) -> ExecutionInputFilesListResponse:
    """List input files for an execution"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url(
        "/api/{version}/execution/{id}/input/files", version=session.version, id=id
    )
    async with session.request(
        "GET", url, data=None, params=filter_none(dict())
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(200): ExecutionInputFilesListResponse}[
                    response.status
                ]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def execution_list_running(
    session: RundeckClient, project: String
) -> ExecutionList:
    """List job executions"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url(
        "/api/{version}/project/{project}/executions/running",
        version=session.version,
        project=project,
    )
    async with session.request(
        "GET", url, data=None, params=filter_none(dict())
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(200): ExecutionList}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def execution_query(
    session: RundeckClient,
    project: String,
    *,
    status_filter: Optional[String] = None,
    abortedby_filter: Optional[String] = None,
    user_filter: Optional[String] = None,
    recent_filter: Optional[String] = None,
    older_filter: Optional[String] = None,
    begin: Optional[String] = None,
    adhoc: Optional[Boolean] = None,
) -> None:
    """Query for Executions based on Job or Execution details"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url(
        "/api/{version}/project/{project}/executions",
        version=session.version,
        project=project,
    )
    async with session.request(
        "GET",
        url,
        data=None,
        params=filter_none(
            dict(
                statusFilter=status_filter,
                abortedbyFilter=abortedby_filter,
                userFilter=user_filter,
                recentFilter=recent_filter,
                olderFilter=older_filter,
                begin=begin,
                adhoc=adhoc,
            )
        ),
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(200): None}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def execution_output_get(
    session: RundeckClient,
    id: String,
    *,
    offset: Optional[String] = None,
    maxlines: Optional[Number] = 5000,
) -> ExecutionOutput:
    """List input files for an execution"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url(
        "/api/{version}/execution/{id}/output", version=session.version, id=id
    )
    async with session.request(
        "GET",
        url,
        data=None,
        params=filter_none(dict(offset=offset, maxlines=maxlines)),
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(200): ExecutionOutput}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )



from async_rundeck.proto.metric import *
from async_rundeck.proto.storage import *
from async_rundeck.proto.user import *
from async_rundeck.proto.execution import *
from async_rundeck.proto.system import *
from async_rundeck.proto.project import *
from async_rundeck.proto.job import *
from async_rundeck.proto.definitions import *

from async_rundeck.proto import _forward_refs



import json
from typing import TypeVar, NewType, NewType

T = TypeVar("T")

Number = float
String = str
Boolean = bool
Integer = int
Object = dict
File = NewType("File", bytes)

__all__ = ["Number", "String", "Boolean", "Integer", "Object", "File"]



# DON'T CHANGE MANUALLY THIS FILE.
# This file is generated from https://github.com/rundeck/rundeck-api-specs
from enum import Enum
from typing import List, Optional
from pydantic import BaseModel, Field, parse_obj_as
from async_rundeck.proto.json_types import (
    Integer,
    Number,
    String,
    Boolean,
    Object,
    File,
)
import json
from enum import Enum
from typing import List, Optional, Union
from pydantic import parse_raw_as, BaseModel, Field
from async_rundeck.proto.json_types import (
    Integer,
    Number,
    String,
    Boolean,
    Object,
    File,
)
from async_rundeck.client import RundeckClient
from async_rundeck.misc import filter_none
from async_rundeck.exceptions import RundeckError, VersionError


async def metric_list(session: RundeckClient) -> Object:
    """List links to enabled Metrics endpoints"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url("/api/{version}/metrics", version=session.version)
    async with session.request(
        "GET", url, data=None, params=filter_none(dict())
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(200): Object}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )



# DON'T CHANGE MANUALLY THIS FILE.
# This file is generated from https://github.com/rundeck/rundeck-api-specs
from enum import Enum
from typing import List, Optional
from pydantic import BaseModel, Field, parse_obj_as
from async_rundeck.proto.json_types import (
    Integer,
    Number,
    String,
    Boolean,
    Object,
    File,
)
import json
from enum import Enum
from typing import List, Optional, Union
from pydantic import parse_raw_as, BaseModel, Field
from async_rundeck.proto.json_types import (
    Integer,
    Number,
    String,
    Boolean,
    Object,
    File,
)
from async_rundeck.client import RundeckClient
from async_rundeck.misc import filter_none
from async_rundeck.exceptions import RundeckError, VersionError
from async_rundeck.proto.definitions import StorageKeyListResponse, File


async def storage_key_get_material(
    session: RundeckClient, key_path: String, accept: String
) -> Union["File", None]:
    """Get key material at the specified PATH"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url(
        "/api/{version}/storage/keys/{keyPath}",
        version=session.version,
        keyPath=key_path,
    )
    async with session.request(
        "GET", url, data=None, params=filter_none(dict())
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(200): File, (404): None}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def storage_key_get_metadata(
    session: RundeckClient, path: String, accept: String
) -> Union[StorageKeyListResponse, None]:
    """List resources at the specified PATH"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url(
        "/api/{version}/storage/keys/{path}", version=session.version, path=path
    )
    async with session.request(
        "GET", url, data=None, params=filter_none(dict())
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(200): StorageKeyListResponse, (404): None}[
                    response.status
                ]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def storage_key_create(
    session: RundeckClient,
    path: String,
    file: "File",
    *,
    content_type: Optional[String] = "application/pgp-keys",
) -> Union[None, None]:
    """Set storage key contents"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url(
        "/api/{version}/storage/keys/{path}", version=session.version, path=path
    )
    async with session.request(
        "POST", url, data=file, params=filter_none(dict())
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(201): None, (409): None}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def storage_key_update(
    session: RundeckClient,
    path: String,
    file: "File",
    *,
    content_type: Optional[String] = "application/pgp-keys",
) -> None:
    """Set storage key contents"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url(
        "/api/{version}/storage/keys/{path}", version=session.version, path=path
    )
    async with session.request(
        "PUT", url, data=file, params=filter_none(dict())
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(200): None}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def storage_key_delete(session: RundeckClient, path: String) -> None:
    """Deletes the file if it exists and returns 204 response."""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url(
        "/api/{version}/storage/keys/{path}", version=session.version, path=path
    )
    async with session.request(
        "DELETE", url, data=None, params=filter_none(dict())
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(204): None}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )



# DON'T CHANGE MANUALLY THIS FILE.
# This file is generated from https://github.com/rundeck/rundeck-api-specs
from enum import Enum
from typing import List, Optional
from pydantic import BaseModel, Field, parse_obj_as
from async_rundeck.proto.json_types import (
    Integer,
    Number,
    String,
    Boolean,
    Object,
    File,
)
import json
from enum import Enum
from typing import List, Optional, Union
from pydantic import parse_raw_as, BaseModel, Field
from async_rundeck.proto.json_types import (
    Integer,
    Number,
    String,
    Boolean,
    Object,
    File,
)
from async_rundeck.client import RundeckClient
from async_rundeck.misc import filter_none
from async_rundeck.exceptions import RundeckError, VersionError
from async_rundeck.proto.definitions import (
    SystemInfo,
    LogStorage,
    IncompleteLogExecutions,
    TakeoverScheduleResponse,
    Job,
    AclList,
    AclPolicyResponse,
    InvalidAclPolicyResponse,
)


class SystemIncompleteLogStorageExecutionsResumeResponse(BaseModel):
    resumed: Optional[Boolean] = Field(alias="resumed")


class SystemExecutionsEnableResponse(BaseModel):
    execution_mode: Optional[String] = Field(alias="executionMode")


class SystemExecutionsDisableResponse(BaseModel):
    execution_mode: Optional[String] = Field(alias="executionMode")


class SchedulerTakeoverRequest(BaseModel):
    server: Optional[Object] = Field(alias="server")
    project: Optional[String] = Field(alias="project")
    job: Optional[Object] = Field(alias="job")


class SystemAclPolicyCreateRequest(BaseModel):
    contents: String = Field(alias="contents")


class SystemAclPolicyUpdateRequest(BaseModel):
    contents: String = Field(alias="contents")


async def system_info_get(session: RundeckClient) -> SystemInfo:
    """Get Rundeck server information and stats"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url("/api/{version}/system/info", version=session.version)
    async with session.request(
        "GET", url, data=None, params=filter_none(dict())
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(200): SystemInfo}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def system_log_storage_info_get(session: RundeckClient) -> LogStorage:
    """Get Log Storage information and stats"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url(
        "/api/{version}/system/logstorage", version=session.version
    )
    async with session.request(
        "GET", url, data=None, params=filter_none(dict())
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(200): LogStorage}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def system_incomplete_log_storage_executions_get(
    session: RundeckClient,
) -> IncompleteLogExecutions:
    """List all executions with incomplete log storage"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url(
        "/api/{version}/system/logstorage/incomplete", version=session.version
    )
    async with session.request(
        "GET", url, data=None, params=filter_none(dict())
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(200): IncompleteLogExecutions}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def system_incomplete_log_storage_executions_resume(
    session: RundeckClient,
) -> SystemIncompleteLogStorageExecutionsResumeResponse:
    """Resume processing incomplete Log Storage uploads"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url(
        "/api/{version}/system/logstorage/incomplete/resume", version=session.version
    )
    async with session.request(
        "POST", url, data=None, params=filter_none(dict())
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {
                    (200): SystemIncompleteLogStorageExecutionsResumeResponse
                }[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def system_executions_enable(
    session: RundeckClient,
) -> SystemExecutionsEnableResponse:
    """Enables executions, allowing adhoc and manual and scheduled jobs to be run"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url(
        "/api/{version}/system/executions/enable", version=session.version
    )
    async with session.request(
        "POST", url, data=None, params=filter_none(dict())
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(200): SystemExecutionsEnableResponse}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def system_executions_disable(
    session: RundeckClient,
) -> SystemExecutionsDisableResponse:
    """Disables executions, preventing adhoc and manual and scheduled jobs from running."""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url(
        "/api/{version}/system/executions/disable", version=session.version
    )
    async with session.request(
        "POST", url, data=None, params=filter_none(dict())
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(200): SystemExecutionsDisableResponse}[
                    response.status
                ]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def system_scheduler_takeover(
    session: RundeckClient,
    *,
    scheduler_takeover_request: Optional[SchedulerTakeoverRequest] = None,
) -> TakeoverScheduleResponse:
    """Tell a Rundeck server in cluster mode to claim all scheduled jobs from another cluster server"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url(
        "/api/{version}/scheduler/takeover", version=session.version
    )
    async with session.request(
        "PUT",
        url,
        data=json.dumps(scheduler_takeover_request)
        if isinstance(scheduler_takeover_request, dict)
        else scheduler_takeover_request.json(),
        params=filter_none(dict()),
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(200): TakeoverScheduleResponse}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def system_scheduled_jobs_for_server(
    session: RundeckClient, uuid: String
) -> List["Job"]:
    """List the scheduled Jobs with their schedule owned by the cluster server with the specified UUID"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url(
        "/api/{version}/scheduler/server/{uuid}/jobs",
        version=session.version,
        uuid=uuid,
    )
    async with session.request(
        "GET", url, data=None, params=filter_none(dict())
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(200): List[Job]}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def system_scheduled_jobs_list(session: RundeckClient) -> List[Job]:
    """List the scheduled Jobs with their schedule owned by the cluster server"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url("/api/{version}/scheduler/jobs", version=session.version)
    async with session.request(
        "GET", url, data=None, params=filter_none(dict())
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(200): List[Job]}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def system_acl_policy_list(session: RundeckClient) -> Union[AclList, None]:
    """List ACL Policies"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url("/api/{version}/system/acl/", version=session.version)
    async with session.request(
        "GET", url, data=None, params=filter_none(dict())
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(200): AclList, (404): None}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def system_acl_policy_get(
    session: RundeckClient, policy_name: String
) -> Union[AclPolicyResponse, None]:
    """Retrieve the YAML texas of the ACL Policy file"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url(
        "/api/{version}/system/acl/{policyName}",
        version=session.version,
        policyName=policy_name,
    )
    async with session.request(
        "GET", url, data=None, params=filter_none(dict())
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(200): AclPolicyResponse, (404): None}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def system_acl_policy_create(
    session: RundeckClient,
    policy_name: String,
    *,
    system_acl_policy_create_request: Optional[SystemAclPolicyCreateRequest] = None,
) -> Union[AclPolicyResponse, None, InvalidAclPolicyResponse]:
    """Create a policy"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url(
        "/api/{version}/system/acl/{policyName}",
        version=session.version,
        policyName=policy_name,
    )
    async with session.request(
        "POST",
        url,
        data=json.dumps(system_acl_policy_create_request)
        if isinstance(system_acl_policy_create_request, dict)
        else system_acl_policy_create_request.json(),
        params=filter_none(dict()),
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {
                    (201): AclPolicyResponse,
                    (409): None,
                    (400): InvalidAclPolicyResponse,
                }[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def system_acl_policy_update(
    session: RundeckClient,
    policy_name: String,
    *,
    system_acl_policy_update_request: Optional[SystemAclPolicyUpdateRequest] = None,
) -> Union[AclPolicyResponse, None]:
    """Update policy"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url(
        "/api/{version}/system/acl/{policyName}",
        version=session.version,
        policyName=policy_name,
    )
    async with session.request(
        "PUT",
        url,
        data=json.dumps(system_acl_policy_update_request)
        if isinstance(system_acl_policy_update_request, dict)
        else system_acl_policy_update_request.json(),
        params=filter_none(dict()),
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(200): AclPolicyResponse, (404): None}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def system_acl_policy_delete(
    session: RundeckClient, policy_name: String
) -> Union[None, None]:
    """Delete policy"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url(
        "/api/{version}/system/acl/{policyName}",
        version=session.version,
        policyName=policy_name,
    )
    async with session.request(
        "DELETE", url, data=None, params=filter_none(dict())
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(204): None, (404): None}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )



# DON'T CHANGE MANUALLY THIS FILE.
# This file is generated from https://github.com/rundeck/rundeck-api-specs
from enum import Enum
from typing import List, Optional
from pydantic import BaseModel, Field, parse_obj_as
from async_rundeck.proto.json_types import (
    Integer,
    Number,
    String,
    Boolean,
    Object,
    File,
)
import json
from enum import Enum
from typing import List, Optional, Union
from pydantic import parse_raw_as, BaseModel, Field
from async_rundeck.proto.json_types import (
    Integer,
    Number,
    String,
    Boolean,
    Object,
    File,
)
from async_rundeck.client import RundeckClient
from async_rundeck.misc import filter_none
from async_rundeck.exceptions import RundeckError, VersionError
from async_rundeck.proto.definitions import Project, Object, File


class ProjectCreateRequest(BaseModel):
    name: Optional[String] = Field(alias="name")
    config: Optional[Object] = Field(alias="config")


class ProjectConfigKeyGetResponse(BaseModel):
    key: Optional[String] = Field(alias="key")
    value: Optional[String] = Field(alias="value")


class ProjectConfigKeySetResponse(BaseModel):
    key: Optional[String] = Field(alias="key")
    value: Optional[String] = Field(alias="value")


class ProjectConfigKeySetRequest(BaseModel):
    value: Optional[String] = Field(alias="value")


class ProjectReadmeGetResponse(BaseModel):
    contents: Optional[String] = Field(alias="contents")


class ReadmeUpdateRequest(BaseModel):
    contents: Optional[String] = Field(alias="contents")


class ProjectMotdGetResponse(BaseModel):
    contents: Optional[String] = Field(alias="contents")


class MotdUpdateRequest(BaseModel):
    contents: Optional[String] = Field(alias="contents")


async def project_list(session: RundeckClient) -> List[Object]:
    """List projects"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url("/api/{version}/projects", version=session.version)
    async with session.request(
        "GET", url, data=None, params=filter_none(dict())
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(200): List[Object]}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def project_create(
    session: RundeckClient, project_create_request: ProjectCreateRequest
) -> Union[Object, None]:
    """Create a new project"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url("/api/{version}/projects", version=session.version)
    async with session.request(
        "POST",
        url,
        data=json.dumps(project_create_request)
        if isinstance(project_create_request, dict)
        else project_create_request.json(),
        params=filter_none(dict()),
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(201): Object, (409): None}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def project_get(session: RundeckClient, project: String) -> Union[Project, None]:
    """Get information about a project"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url(
        "/api/{version}/project/{project}", version=session.version, project=project
    )
    async with session.request(
        "GET", url, data=None, params=filter_none(dict())
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(200): Project, (404): None}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def project_delete(session: RundeckClient, project: String) -> None:
    """Delete project"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url(
        "/api/{version}/project/{project}", version=session.version, project=project
    )
    async with session.request(
        "DELETE", url, data=None, params=filter_none(dict())
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(204): None}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def project_config_get(
    session: RundeckClient, project: String
) -> Union[Object, None]:
    """Get project config"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url(
        "/api/{version}/project/{project}/config",
        version=session.version,
        project=project,
    )
    async with session.request(
        "GET", url, data=None, params=filter_none(dict())
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(200): Object, (404): None}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def project_config_update(
    session: RundeckClient, project: String, project_config_update_request: Object
) -> None:
    """Update project config"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url(
        "/api/{version}/project/{project}/config",
        version=session.version,
        project=project,
    )
    async with session.request(
        "PUT",
        url,
        data=json.dumps(project_config_update_request)
        if isinstance(project_config_update_request, dict)
        else project_config_update_request.json(),
        params=filter_none(dict()),
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(200): None}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def project_config_key_get(
    session: RundeckClient, project: String, key: String
) -> ProjectConfigKeyGetResponse:
    """Get project config key"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url(
        "/api/{version}/project/{project}/config/{key}",
        version=session.version,
        project=project,
        key=key,
    )
    async with session.request(
        "GET", url, data=None, params=filter_none(dict())
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(200): ProjectConfigKeyGetResponse}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def project_config_key_set(
    session: RundeckClient,
    project: String,
    key: String,
    project_config_key_set_request: ProjectConfigKeySetRequest,
) -> ProjectConfigKeySetResponse:
    """Get project config key"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url(
        "/api/{version}/project/{project}/config/{key}",
        version=session.version,
        project=project,
        key=key,
    )
    async with session.request(
        "PUT",
        url,
        data=json.dumps(project_config_key_set_request)
        if isinstance(project_config_key_set_request, dict)
        else project_config_key_set_request.json(),
        params=filter_none(dict()),
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(200): ProjectConfigKeySetResponse}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def project_config_key_delete(
    session: RundeckClient, project: String, key: String
) -> None:
    """Delete project config key"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url(
        "/api/{version}/project/{project}/config/{key}",
        version=session.version,
        project=project,
        key=key,
    )
    async with session.request(
        "DELETE", url, data=None, params=filter_none(dict())
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(204): None}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def project_jobs_export(
    session: RundeckClient,
    project: String,
    *,
    format: Optional[String] = "xml",
    idlist: Optional[String] = None,
    group_path: Optional[String] = None,
    job_filter: Optional[String] = None,
) -> String:
    """Export the job definitions in XML or YAML formats."""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url(
        "/api/{version}/project/{project}/jobs/export",
        version=session.version,
        project=project,
    )
    async with session.request(
        "GET",
        url,
        data=None,
        params=filter_none(
            dict(
                format=format, idlist=idlist, groupPath=group_path, jobFilter=job_filter
            )
        ),
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(200): String}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def project_jobs_import(
    session: RundeckClient,
    project: String,
    file: "File",
    *,
    content_type: Optional[String] = "application/xml",
    accept: Optional[String] = "application/xml",
    file_format: Optional[String] = "xml",
    dupe_option: Optional[String] = "create",
    uuid_option: Optional[String] = "preserve",
) -> "File":
    """Import job definitions in XML or YAML formats."""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url(
        "/api/{version}/project/{project}/jobs/import",
        version=session.version,
        project=project,
    )
    async with session.request(
        "POST",
        url,
        data=file,
        params=filter_none(
            dict(fileFormat=file_format, dupeOption=dupe_option, uuidOption=uuid_option)
        ),
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(200): File}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def project_archive_import(
    session: RundeckClient,
    project: String,
    file: "File",
    *,
    job_uuid_option: Optional[String] = "remove",
    import_executions: Optional[Boolean] = None,
    import_config: Optional[Boolean] = None,
    import_a_c_l: Optional[Boolean] = None,
) -> None:
    """Import project archive."""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url(
        "/api/{version}/project/{project}/import",
        version=session.version,
        project=project,
    )
    async with session.request(
        "PUT",
        url,
        data=file,
        params=filter_none(
            dict(
                jobUuidOption=job_uuid_option,
                importExecutions=import_executions,
                importConfig=import_config,
                importACL=import_a_c_l,
            )
        ),
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(200): None}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def project_archive_export_sync(
    session: RundeckClient,
    project: String,
    *,
    execution_ids: Optional[Boolean] = None,
    export_all: Optional[Boolean] = True,
    export_jobs: Optional[Boolean] = None,
    export_executions: Optional[Boolean] = None,
    export_configs: Optional[Boolean] = None,
    export_readmes: Optional[Boolean] = None,
    export_acls: Optional[Boolean] = None,
) -> "File":
    """Export archive of project synchronously"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url(
        "/api/{version}/project/{project}/export",
        version=session.version,
        project=project,
    )
    async with session.request(
        "GET",
        url,
        data=None,
        params=filter_none(
            dict(
                executionIds=execution_ids,
                exportAll=export_all,
                exportJobs=export_jobs,
                exportExecutions=export_executions,
                exportConfigs=export_configs,
                exportReadmes=export_readmes,
                exportAcls=export_acls,
            )
        ),
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(200): File}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def project_readme_get(
    session: RundeckClient, project: String
) -> Union[ProjectReadmeGetResponse, None]:
    """Get the readme.md contents"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url(
        "/api/{version}/project/{project}/readme.md",
        version=session.version,
        project=project,
    )
    async with session.request(
        "GET", url, data=None, params=filter_none(dict())
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(200): ProjectReadmeGetResponse, (404): None}[
                    response.status
                ]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def project_readme_put(
    session: RundeckClient, project: String, readme_update_request: ReadmeUpdateRequest
) -> None:
    """Create or modify project README.md"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url(
        "/api/{version}/project/{project}/readme.md",
        version=session.version,
        project=project,
    )
    async with session.request(
        "PUT",
        url,
        data=json.dumps(readme_update_request)
        if isinstance(readme_update_request, dict)
        else readme_update_request.json(),
        params=filter_none(dict()),
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(200): None}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def project_readme_delete(session: RundeckClient, project: String) -> None:
    """Delete project README.md"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url(
        "/api/{version}/project/{project}/readme.md",
        version=session.version,
        project=project,
    )
    async with session.request(
        "DELETE", url, data=None, params=filter_none(dict())
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(204): None}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def project_motd_get(
    session: RundeckClient, project: String
) -> Union[ProjectMotdGetResponse, None]:
    """Get the readme.md contents"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url(
        "/api/{version}/project/{project}/motd.md",
        version=session.version,
        project=project,
    )
    async with session.request(
        "GET", url, data=None, params=filter_none(dict())
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(200): ProjectMotdGetResponse, (404): None}[
                    response.status
                ]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def project_motd_put(
    session: RundeckClient, project: String, motd_update_request: MotdUpdateRequest
) -> None:
    """Create or modify project MOTD.md"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url(
        "/api/{version}/project/{project}/motd.md",
        version=session.version,
        project=project,
    )
    async with session.request(
        "PUT",
        url,
        data=json.dumps(motd_update_request)
        if isinstance(motd_update_request, dict)
        else motd_update_request.json(),
        params=filter_none(dict()),
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(200): None}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )


async def project_motd_delete(session: RundeckClient, project: String) -> None:
    """Delete project motd.md"""
    if session.version < 26:
        raise VersionError(
            f"Insufficient api version error, Required >{session.version}"
        )
    url = session.format_url(
        "/api/{version}/project/{project}/motd.md",
        version=session.version,
        project=project,
    )
    async with session.request(
        "DELETE", url, data=None, params=filter_none(dict())
    ) as response:
        obj = await response.text()
        if response.ok:
            try:
                response_type = {(204): None}[response.status]
                if response_type is None:
                    return None
                elif response_type is String:
                    return obj
                else:
                    return parse_raw_as(response_type, obj)
            except KeyError:
                raise RundeckError(
                    f"Unknwon response code: {session.url}({response.status})"
                )
        else:
            raise RundeckError(
                f"Connection diffused: {session.url}({response.status})\n{obj}"
            )



from pydantic import BaseModel
from async_rundeck.proto import definitions
import inspect

for name, model in inspect.getmembers(definitions, inspect.isclass):
    if not issubclass(model, BaseModel):
        continue

    model.update_forward_refs()



from ctypes import Union
from typing import Any, Dict, List, Literal, Optional, Tuple
from aiohttp import FormData

from pydantic import parse_raw_as
from async_rundeck.client import RundeckClient

from async_rundeck.proto.definitions import (
    Execution,
    ExecutionList,
    Job,
    JobBulkOperationResponse,
    JobInputFileInfo,
    Project,
)
from async_rundeck import proto
from async_rundeck.exceptions import VersionError, RundeckError
from async_rundeck.proto.job import JobScheduleBulkDisableRequest


class Rundeck:
    def __init__(
        self,
        url: str = None,
        token: str = None,
        username: str = None,
        password: str = None,
        api_version: int = None,
    ) -> None:
        self.client = RundeckClient(url, token, username, password, api_version)

    async def __aenter__(self):
        await self.client.__aenter__()
        return self

    async def __aexit__(self, *args):
        await self.client.__aexit__(*args)

    async def list_project(self) -> List[Project]:
        """List the existing projects on the server.

        Returns
        -------
        List[Project]
            List of project informations
        """
        return await proto.project_list(self.client)

    async def create_project(
        self, name: str, config: Dict[str, str] = None
    ) -> Optional[Project]:
        """Create a new project

        Parameters
        ----------
        name : str
            name of the project
        config : Dict[str, Any], optional
            configuration of the project, by default None

        Returns
        -------
        Optional[Project]
            Created information about a project
        """
        project = await proto.project_create(
            self.client, proto.ProjectCreateRequest(name=name, config=config)
        )
        if project is None:
            return project
        else:
            return Project.parse_obj(project)

    async def delete_project(self, name: str) -> None:
        """Delete an existing projects on the server. Requires 'delete' authorization.

        Parameters
        ----------
        name : str
            project name
        """
        await proto.project_delete(self.client, project=name)

    async def get_project(self, name: str) -> Optional[Project]:
        """Get a information about a project.

        Parameters
        ----------
        name : str
            name of the project

        Returns
        -------
        Optional[Project]
            Project information
        """
        await proto.project_get(self.client, project=name)

    async def get_project_config(self, name: str) -> Optional[Dict[str, Any]]:
        """Get the configuration of a project.

        Parameters
        ----------
        name : str
            name of the project

        Returns
        -------
        Optional[Dict[str, Any]]
            Project configuration
        """
        return await proto.project_config_get(self.client, project=name)

    async def get_project_config_item(
        self, name: str, key: str
    ) -> Optional[Tuple[str, str]]:
        """Get a configuration item of a project.

        Parameters
        ----------
        name : str
            name of the project
        key : str
            key of the configuration item

        Returns
        -------
        Optional[Tuple[str, str]]
            Project configuration item
        """
        kv = await proto.project_config_key_get(self.client, project=name, key=key)
        return kv.key, kv.value

    async def update_project_config(self, name: str, config: Dict[str, Any]) -> None:
        """Update the configuration of a project.

        Parameters
        ----------
        name : str
            name of the project
        config : Dict[str, Any]
            Project configuration
        """
        await proto.project_config_update(
            self.client, project=name, project_config_update_request=config
        )

    async def update_project_config_item(
        self, name: str, key: str, value: Any
    ) -> Tuple[Optional[str], Optional[str]]:
        """Update a configuration item of a project.

        Parameters
        ----------
        name : str
            name of the project
        key : str
            key of the configuration item
        value : Any
            value of the configuration item

        Returns
        -------
        Tuple[Optional[str], Optional[str]]
            Updated key/value pairs
        """
        response = await proto.project_config_key_set(
            self.client,
            project=name,
            key=key,
            project_config_key_set_request=proto.ProjectConfigKeySetRequest(
                value=value
            ),
        )
        return (response.key, response.value)

    async def delete_project_config_item(self, name: str, key: str) -> None:
        """Delete a configuration item of a project.

        Parameters
        ----------
        name : str
            name of the project
        key : str
            key of the configuration item
        """
        await proto.project_config_key_delete(self.client, project=name, key=key)

    # Jobs functions
    async def get_job_by_name(self, project: str, name: str) -> Optional[Job]:
        """Get a job by name.

        Parameters
        ----------
        name : str
            name of the job

        Returns
        -------
        Optional[Job]
            Job information
        """
        jobs = await self.list_jobs(project)
        for job in jobs:
            if job.name == name:
                return job

    async def list_jobs(
        self,
        project: str,
        *,
        id_list: Optional[str] = None,
        group_path: Optional[str] = "*",
        job_filter: Optional[str] = None,
        job_exact_filter: Optional[str] = None,
        group_path_exact: Optional[str] = None,
        scheduled_filter: Optional[bool] = None,
        server_node_uuid_filter: Optional[str] = None,
        max: Optional[int] = None,
        offset: Optional[int] = None,
    ) -> List[proto.Job]:
        """List the existing jobs on the server.

        Parameters
        ----------
        project : str
            project name
        id_list : Optional[str], optional
            comma separated list of job ids, by default None
        group_path : Optional[str], optional
            specify a group or partial group path to include all jobs within that group path.
            Set to the special value "-" to match the top level jobs only.
            by default "*", which matches all groups.
        job_filter : Optional[str], optional
             specify a filter for the job Name.
             Matches any job name that contains this value., by default None
        job_exact_filter : Optional[str], optional
            specify an exact job name to match. by default None
        group_path_exact : Optional[str], optional
            group path exact, by default None
        scheduled_filter : Optional[bool], optional
            true/false specify whether to return only scheduled or only not scheduled jobs.
            by default None
        server_node_uuid_filter : Optional[str], optional
            Value: a UUID.
            In cluster mode, use to select scheduled jobs assigned to the server with given UUID.
            by default None
        max: Optional[int]
            limit the maximum amount of results to be received.
            Not implemeneted yet.
        offset: Optional[int]
            use in conjunction with max to paginate the result set.
            Not implemeneted yet.

        Returns
        -------
        List[proto.Job]
            List of job informations
        """
        if max is not None or offset is not None:
            raise NotImplementedError("`max` and `offset` are not implemented yet.")
        return await proto.job_list(
            self.client,
            project=project,
            id_list=id_list,
            group_path=group_path,
            job_filter=job_filter,
            job_exact_filter=job_exact_filter,
            group_path_exact=group_path_exact,
            scheduled_filter=scheduled_filter,
            server_node_uuid_filter=server_node_uuid_filter,
        )

    async def execute_job(
        self,
        job_id: str,
        *,
        arg_string: Optional[str] = None,
        log_level: Optional[proto.Loglevel] = None,
        as_user: Optional[str] = None,
        filter: Optional[str] = None,
        run_at_time: Optional[str] = None,
        options: Optional[Dict[str, Any]] = None,
    ) -> proto.Execution:
        """Run a job specified by job_id.

        Parameters:
            job_id: str
                Job id
            argString: str, optional
                argument string to pass to the job, of the form: -opt value -opt2 value ....
            loglevel: LogLevel, optional
                argument specifying the loglevel to use, one of: 'DEBUG','VERBOSE','INFO','WARN','ERROR'
            asUser  str, optional
                specifies a username identifying the user who ran the job. Requires runAs permission.
            Node  str, optional
                filter parameters as described under Using Node Filters
            filter  str, optional
                can be a node filter string.
            runAtTime: str, optional
                 Specify a time to run the job (API v18 or later).
            option. str, optional
                OPTNAME: Option value for option named OPTNAME. If any option.OPTNAME parameters are specified, the argString value is ignored (API v18 or later).
        """
        return await proto.job_execution_run(
            self.client,
            id=job_id,
            request=proto.ExecuteJobRequest(
                arg_string=arg_string,
                log_level=log_level,
                as_user=as_user,
                filter=filter,
                run_at_time=run_at_time,
                options=options,
            ),
        )

    async def retry_job(
        self, job_id: str, execution_id: int, *, retry_count: int
    ) -> proto.Execution:
        """Retry a job specified by job_id.

        Parameters
        ----------
        job_id : str
            Job id
        execution_id : str
            Execution id
        retry_count : int
            Number of maximum retries
        """
        return await proto.job_retry_execution(
            self.client,
            job_id=job_id,
            execution_id=execution_id,
            request=proto.RetryExecutionRequest(retry_count=retry_count),
        )

    async def export_jobs(
        self,
        project: str,
        *,
        format: Literal["xml", "yaml"] = "xml",
        idlist: Optional[str] = None,
        group_path: Optional[str] = None,
        job_filter: Optional[str] = None,
    ) -> Optional[str]:
        """Export jobs in a project to a file.

        Parameters
        ----------
        project : str
            Project name
        format : "xml"|"yaml", optional
            can be "xml" or "yaml" to specify the output format. Default is "xml"
        The following parameters can also be used to narrow down the result set.

        idlist: str, optional
            specify a comma-separated list of Job IDs to export
        groupPath: str, optional
            specify a group or partial group path to include all jobs within that group path.
        jobFilter: str, optional
            specify a filter for the job Name
        """
        return await proto.project_jobs_export(
            self.client,
            project=project,
            format=format,
            idlist=idlist,
            group_path=group_path,
            job_filter=job_filter,
        )

    async def import_jobs(
        self,
        project: str,
        file: str,
        *,
        content_type: Optional[
            Literal[
                # "x-www-form-urlencoded",
                # "multipart/form-data",
                "application/xml",
                "application/yaml",
            ]
        ] = "x-www-form-urlencoded",
        file_format: Optional[Literal["xml", "yaml"]] = "xml",
        dupe_option: Optional[Literal["skip", "create", "update"]] = "create",
        uuid_option: Optional[Literal["preserve", "remove"]] = "preserve",
    ) -> Optional[Dict[Literal["succeeded", "failed", "skipped"], List[Any]]]:
        """Export jobs in a project to a file.

        Parameters
        ----------
        project : str
            Project name

        content_type: Literal[""], optional
            [Not implemeneted] Content-Type: x-www-form-urlencoded, with a xmlBatch
                request parameter containing the input content
            [Not implemeneted] Content-Type: multipart/form-data multipart MIME request part
                named xmlBatch containing the xml content.
            Content-Type: application/xml, request body is the Jobs XML formatted job definition
            Content-Type: application/yaml, request body is the Jobs YAML formatted job definition

        file_format : Literal["xml", "yaml"], optional
            can be "xml" or "yaml" to specify the input format,
            if multipart of form input is sent. Default is "xml"
        dupe_option:
            A value to indicate the behavior when importing jobs which already exist.
            Value can be "skip", "create", or "update". Default is "create".
        uuid_option:
            Whether to preserve or remove UUIDs from the imported jobs. Allowed values (since V9):
            - preserve: Preserve the UUIDs in imported jobs.
                This may cause the import to fail if the UUID is already used. (Default value).
            - remove: Remove the UUIDs from imported jobs.
                Allows update/create to succeed without conflict on UUID.

        Returns
        -------
        Optional[Dict[Literal["succeeded", "failed", "skipped"], List[Any]]]
            A set of status results.
        """
        if content_type == "multipart/form-data":
            data = FormData()
            data.add_field("xmlBatch", file, content_type=content_type)
        elif content_type == "application/xml":
            data = file
        elif content_type == "application/yaml":
            data = file
        else:
            raise ValueError(f"Unsupported content_type: {content_type}")
        with self.client.context_options(
            {
                "headers": {
                    "Accept": "application/json",
                    "Content-Type": content_type,
                }
            }
        ):
            session = self.client
            if session.version < 26:
                raise VersionError(
                    f"Insufficient api version error, Required >{session.version}"
                )
            url = session.format_url(
                "/api/{version}/project/{project}/jobs/import",
                version=session.version,
                project=project,
            )
            async with session.request(
                "POST",
                url,
                data=data,
                params=dict(
                    fileFormat=file_format,
                    dupeOption=dupe_option,
                    uuidOption=uuid_option,
                ),
            ) as response:
                obj = await response.text()
                if response.ok:
                    try:
                        response_type = {
                            (200): Dict[
                                Literal["succeeded", "failed", "skipped"], List[Any]
                            ]
                        }[response.status]
                        if response_type is None:
                            return None
                        else:
                            return parse_raw_as(response_type, obj)
                    except KeyError:
                        raise RundeckError(
                            f"Unknwon response code: {session.url}({response.status})"
                        )
                else:
                    raise RundeckError(
                        f"Connection diffused: {session.url}({response.status})\n{obj}"
                    )

    async def upload_file_for_job(
        self, job_id: str, option_name: str, file: bytes, file_name: str = None
    ) -> Optional[str]:
        """Upload file for job.

        Parameters
        ----------
        job_id : str
            Job id
        option_name : str
            name of option
        file : bytes
            byte array of the file

        Returns
        -------
        Optional[str]
            id of the uploaded file
        """
        with self.client.context_options(
            {
                "headers": {
                    "Accept": "application/json",
                    "Content-Type": "application/octet-stream",
                }
            }
        ):
            response = await proto.job_input_file_upload(
                self.client, job_id, option_name, file_name=file_name, file=file
            )
            return response.options.get(option_name, None)

    async def list_files_for_job(self, job_id: str) -> List[JobInputFileInfo]:
        """List files for job.

        Parameters
        ----------
        job_id : str

        Returns
        -------
        List[JobInputFileInfo]
            information of uploaded files.
        """
        response = await proto.job_input_file_list(self.client, job_id)
        return response.files

    async def enable_scheduling_job(self, job_id: str) -> bool:
        """Enable scheduling for job.

        Parameters
        ----------
        job_id : str

        Response
        ----------
        bool
            True if succceeded
        """
        response = await proto.job_schedule_enable(self.client, job_id)
        return response.success

    async def disable_scheduling_job(self, job_id: str) -> bool:
        """Disable scheduling for job.

        Parameters
        ----------
        job_id : str

        Response
        ----------
        bool
            True if succceeded
        """
        response = await proto.job_schedule_disable(self.client, job_id)
        return response.success

    async def enable_scheduling_jobs(
        self, job_ids: List[str]
    ) -> JobBulkOperationResponse:
        """Enable scheduling for jobs.

        Parameters
        ----------
        job_ids : st[str]

        Response
        ----------
        JobBulkOperationResponse
            Bulk operation results
        """
        return await proto.job_schedule_bulk_enable(
            self.client, JobScheduleBulkDisableRequest(ids=job_ids)
        )

    async def disable_scheduling_jobs(
        self, job_ids: List[str]
    ) -> JobBulkOperationResponse:
        """Disable scheduling for jobs.

        Parameters
        ----------
        job_ids : List[str]

        Response
        ----------
        JobBulkOperationResponse
            Bulk operation results
        """
        return await proto.job_schedule_bulk_disable(
            self.client, JobScheduleBulkDisableRequest(ids=job_ids)
        )

    # Executions
    async def get_execution(
        self,
        execution_id: str,
        # *,
        # status: Optional[Literal["suceeded", "failed", "aborted", "running"]] = None,
        # max: Optional[int] = None,
        # offset: Optional[int] = None,
    ) -> Execution:
        """Get the list of executions for a job.

        Parameters
        ----------
        execution_id : str
            Execution id
        status: Literal["suceeded", "failed", "aborted", "running"], optional
            the status of executions you want to be returned.
            Must be one of "succeeded", "failed", "aborted", or "running".
            If this parameter is blank or unset, include all executions.
        max: int, optional
            indicate the maximum number of results to return.
            If unspecified, all results will be returned.
        offset: int, optional
            indicate the 0-indexed offset for the first result to return.

        Returns
        -------
        List[proto.Execution]
            List of executions
        """
        return await proto.execution_status_get(self.client, id=execution_id)

    async def delete_execution(self, execution_id: str) -> None:
        """Delete an execution.

        Parameters
        ----------
        execution_id : str
            Execution id
        """
        return await proto.execution_delete(self.client, execution_id)

    async def list_running_executions(self, project: str) -> ExecutionList:
        """List running executions.

        Parameters
        ----------
        project : str
            Project name

        Returns
        -------
        ExecutionList
            executions
        """
        return await proto.execution_list_running(self.client, project=project)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['async_rundeck', 'async_rundeck.proto']

package_data = \
{'': ['*']}

install_requires = \
['aiohttp>=3.8.1,<4.0.0',
 'pydantic>=1.9.0,<2.0.0',
 'pytest-asyncio>=0.18.3,<0.19.0']

setup_kwargs = {
    'name': 'async-rundeck',
    'version': '0.1.3',
    'description': 'Asynchronous rundeck API client',
    'long_description': '[![codecov](https://codecov.io/gh/elda27/async_rundeck/branch/main/graph/badge.svg?token=wo3QBnKsKX)](https://codecov.io/gh/elda27/async_rundeck)\n\n# Asynchronous rundeck API client\n\nThis is a rundeck API client implemeneted by aio-http and pydantic.\nAll API parameters and responses are annotated and user-friendly!\n\n## Installation\n\n```bash\npip install async-rundeck\n```\n\n## How to use\n\n```python\nimport asyncio\nfrom async_rundeck import Rundeck\n\nasync def main():\n    Rundeck(\n        url=rundeck_service, username="admin", password="admin", api_version=41\n    )\n    project_name = uuid4().hex\n    await rundeck.create_project(project_name)\n\n    job_content = (root_dir / "resource" / "Test_job.xml").read_text()\n    # Import job\n    status = await rundeck.import_jobs(\n        project_name,\n        job_content,\n        content_type="application/xml",\n        uuid_option="remove",\n    )\n    if len(status["succeeded"]) != 1:\n      print("Failed to import job")\n      return\n\n    jobs = await rundeck.list_jobs(project_name)\n\n    # Execute job\n    execution = await rundeck.execute_job(jobs[0].id)\n    assert execution is not None\n\nasyncio.run_until_complete(main())\n```\n\n## Features\n\nThe items checked in the following list are implemented.\n\n- [ ] System Info\n- [ ] List Metrics\n  - [ ] Metrics Links\n  - [ ] Metrics Data\n  - [ ] Metrics Healthcheck\n  - [ ] Metrics Threading\n  - [ ] Metrics Ping\n- [ ] User Profile\n- [ ] Log Storage\n- [ ] Execution Mode\n- [ ] Cluster Mode\n- [ ] ACLs\n- [ ] Jobs\n  - [x] List job\n  - [x] Run job\n  - [x] Import job from file\n  - [x] Export job from file\n- [ ] Executions\n  - [x] Get Executions for a Job\n  - [ ] Delete all Executions for a Job\n  - [x] Listing Running Executions\n  - [ ] Execution Info\n  - [x] Upload files for an Execution.\n  - [x] List Input Files for an Execution\n  - [x] Delete an Execution\n  - [ ] Bulk Delete Executions\n  - [ ] Execution Query\n  - [ ] Execution State\n  - [ ] Execution Output\n  - [ ] Execution Output with State\n  - [ ] Aborting Executions\n- [ ] Adhoc\n- [ ] Key Storage\n  - [ ] Upload keys\n  - [ ] List keys\n  - [ ] Get Key Metadata\n  - [ ] Get Key Contents\n  - [ ] Delete Keys\n- [ ] Projects\n  - [x] Listing Projects\n  - [x] Project Creation\n  - [x] Getting Project Info\n  - [x] Project Deletion\n  - [x] Project Configuration\n  - [x] Project Configuration Keys\n  - [ ] Project Archive Export\n  - [ ] Project Archive Export Async\n  - [ ] Project Archive Export Status\n  - [ ] Project Archive Import\n  - [ ] Updating and Listing Resources for a Project\n  - [ ] Project Readme File\n  - [ ] Project ACLs\n- [ ] Listing History\n- [ ] Resources/Nodes\n- [ ] SCM\n',
    'author': 'elda27',
    'author_email': 'kaz.birdstick@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/elda27/async_rundeck',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
